package jp.cqpub.transistor;

import java.io.UnsupportedEncodingException;
import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.SeekBar.OnSeekBarChangeListener;

//
//	SimpleWiFi main Activity
//
public class SimpleWiFi extends Activity
{
//	Common variables
	String ModuleAdrs = "192.168.11.5";	// WiFi module IP address (edit it please)
	int	TextSize = 40;	// Temperature & Voltage text size (edit it please)
	
	LinearLayout layout1;	// Layout object
	SeekBar seekBar1, seekBar2, seekBar3;// Seek bar object
	TextView text1, text2;	// Text view object
	TextView text3, text4;	// Text view object for space
	Udp UdpObj;	// UDP object
	Thread DispThread;	// Display thread object
	volatile boolean abortflag;	// Thread abort flag
	Handler hHandler;	// Drawing handler
	int	Servo1, Servo2, Servo3;	// Servo1 & 2 & 3 value
	//
	//	onCreate
	//
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);	// Do default call back procedures first 
		hHandler = new Handler();	// Create UI handler
		abortflag = false;	// Clear abort flag first
		Servo1 = 50;	// Set default servo1 value
		Servo2 = 50;	// Set default servo2 value
		Servo3 = 50;	// Set default servo3 value
		
		layout1 = new LinearLayout(this);	// Create linear layout object
		layout1.setOrientation(LinearLayout.VERTICAL);	// Set vertical arrangement
    setContentView(layout1);	// Attach layout object to main view

    text1 = new TextView(this);	// Create text of 1st item
		text1.setTextSize(TextSize);	// Set text size 50
		text1.setTextColor(Color.CYAN);	// Set CYAN color
		layout1.addView(text1);	// Attach 1st text object

    text2 = new TextView(this);	// Create text of 1st item
		text2.setTextSize(TextSize);	// Set text size 50
		text2.setTextColor(Color.CYAN);	// Set CYAN color
		layout1.addView(text2);	// Attach 1st text object

		seekBar1 = new SeekBar(this);	// Create Seek Bar object
		seekBar1.setMax(99);	// Set maximum number of this seek bar
		seekBar1.setProgress(50);	// Place sum to center first
		layout1.addView(seekBar1);	// Attach seek bar 1

		text3 = new TextView(this);	// Create text view for spacer
		text3.setTextSize(20);	// Set spacer height
		layout1.addView(text3);	// Attach text object for spacer

		seekBar2 = new SeekBar(this);	// Create Seek Bar object
		seekBar2.setMax(99);	// Set maximum number of this seek bar
		seekBar2.setProgress(50);	// Place sum to center first
		layout1.addView(seekBar2);	// Attach seek bar 2

		text4 = new TextView(this);	// Create text view for spacer
		text4.setTextSize(20);	// Set spacer height
		layout1.addView(text4);	// Attach text object for spacer

		seekBar3 = new SeekBar(this);	// Create Seek Bar object
		seekBar3.setMax(99);	// Set maximum number of this seek bar
		seekBar3.setProgress(50);	// Place sum to center first
		layout1.addView(seekBar3);	// Attach seek bar 2
		//
		//	Seek Bar 1 event listener
		//
		seekBar1.setOnSeekBarChangeListener(new OnSeekBarChangeListener()
		{
			@Override
			public void onStartTrackingTouch(SeekBar seekBar){}	// Do nothing
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch)
			{
				Servo1 = seekBar1.getProgress();	// Get sum position
				byte [] senddata = GetSendData().getBytes();	// Build send data
				UdpObj.Send(senddata, senddata.length);	// UDP packet send
			}
			@Override
			public void onStopTrackingTouch(SeekBar seekBar){}	// Do nothing
		});
		//
		//	Seek Bar 2 event listener
		//
		seekBar2.setOnSeekBarChangeListener(new OnSeekBarChangeListener()
		{
			@Override
			public void onStartTrackingTouch(SeekBar seekBar){}	// Do nothing
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch)
			{
				Servo2 = seekBar2.getProgress();	// Get sum position
				byte [] senddata = GetSendData().getBytes();	// Build send data
				UdpObj.Send(senddata, senddata.length);	// UDP packet send
			}
			@Override
			public void onStopTrackingTouch(SeekBar seekBar){}	// Do nothing
		});
		//
		//	Seek Bar 3 event listener
		//
		seekBar3.setOnSeekBarChangeListener(new OnSeekBarChangeListener()
		{
			@Override
			public void onStartTrackingTouch(SeekBar seekBar){}	// Do nothing
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch)
			{
				Servo3 = seekBar3.getProgress();	// Get sum position
				byte [] senddata = GetSendData().getBytes();	// Build send data
				UdpObj.Send(senddata, senddata.length);	// UDP packet send
			}
			@Override
			public void onStopTrackingTouch(SeekBar seekBar){}	// Do nothing
		});
	}
	//
	//	Create Temperature & Voltage display thread
	//
	private void StartDisplayThread()
	{
		DispThread = new Thread(new Runnable()	// Create thread object
		{
			@Override
			public void run()
			{
				float	temp;	// Temperature work
				float	volt;	// volt work
				for (;!abortflag;)	// Repeat forever until abortflag is set
				{
					if(UdpObj.GetQueueCount() != 0)// Packet has come?
					{	// Yes, some UDP packet has come
						String receivestr;	// Prepare string work
						byte [] senddata = new byte[UdpObj.GetLength()];	// Create receive buffer
						UdpObj.GetData(senddata);	// Get UDP packet from queue
						try
						{
							receivestr = new String(senddata, "US-ASCII");	// Convert byte array to string
							try
							{
								temp = Integer.parseInt(GetNumberString(receivestr, 'D'));	// Get 'D' header number
								volt = Integer.parseInt(GetNumberString(receivestr, 'E'));	// Get 'E' header number							
							}
							catch (NumberFormatException e){temp = 0; volt = 0;}
							final String str1 = String.format("%2.1f", (float) temp * 330 / 1024 - 50) + "";
							final String str2 = String.format("%1.2f", (float) volt * 3.3 / 1024) + "V";
							hHandler.post( new Runnable() {public void run()	// Send Handler message
							{
								text1.setText(str1);	// Post text1 draw command into message queue
								text2.setText(str2);	// Post text2 draw command into message queue
							}});
						}
						catch (UnsupportedEncodingException e){}	// Error trap but no error happens
					}
					try
					{
						Thread.sleep(100);	// Sleep a while to give execution time to anther thread 
					}
					catch (InterruptedException e){	}// Ignore error(never happens)
				}
				abortflag = false;	// Clear abort flag for thread exit indication
			}});
		DispThread.start();	// Start thread just after definition
	}
	
	//
	//	onResume() process
	//
	@Override
	public void onResume()
	{
		super.onResume();	// Do default process
		UdpObj = new Udp(ModuleAdrs, 9750, 9750);	// Create UDP object
		StartDisplayThread();	// Start display temperature & voltage display thread
	}

	//
	//	onPause() process
	//
	@Override
	public void onPause()
	{
		super.onPause();	// Do default process
		if (UdpObj != null)	// UDP object exists?
			UdpObj.Abort();	// Terminate UDP handler thread
		abortflag = true;	// Set display thread abort flag
		while(abortflag);	// Abort confirmation
	}
	//
	//	Get number string from integer
	//
	private String GetDigit(int num)
	{
		int div = 10000000;	// Power of ten
		boolean nonzero = false;	// Store index
		byte dgt;	// Digit
		String result = "";
		while (div >= 1)	// until 10
		{
			dgt = (byte) (num / div); // Set digit
			if(nonzero == true || dgt != 0)// Leading zero supress?
			{
				result = result + String.valueOf(dgt); // Set digit
				nonzero = true;
			}
			num %= div; // Get modulo
			div /= 10; // Divided by ten
			if(div == 1)	// Only one digit?
				nonzero = true;
		}
		return result;	// Return with length
	}
	//
	//	Build UDP packet contents
	//
	private String GetSendData()
	{
		return "A" + GetDigit(Servo1)	// Set A000,
		+ ",B" + GetDigit(Servo2)	// Set B000,
		+ ",C" + GetDigit(Servo3)	+ ",";// Set C000,
	}
	//
	//	Extract number string from received packet
	//
	private String GetNumberString(String str, char ch)
	{
		int index = str.indexOf(ch);	// Get 'D' or 'E' position
		if(index < 0)	// Header char not found?
			return "0";	//	Return with "0"
		str = str.substring(index + 1);	// Skip header char'
		index = str.indexOf(',');	// Find comma
		if(index < 0)	// Comma not found?
			return "0";	//	Return with invalid number
		return str.substring(0, index);
	}
}